/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file Base64.h
 *    Encode/Decode base64 strings.
 *
 * @author Tim Langner
 * @date   2011/12/06
 */

#ifndef _MIRA_BASE64_H_
#define _MIRA_BASE64_H_

#include <string>

#include <platform/Types.h>
#include <utils/Buffer.h>

namespace mira {

///////////////////////////////////////////////////////////////////////////////

/**
 * Base 64 encodes a string.
 * @param[in] data Data to be encoded
 * @return base 64 encoded string
 */
MIRA_BASE_EXPORT std::string base64Encode(const std::string& data);

/**
 * Base 64 encodes data in a buffer.
 * @param[in] data Data to be encoded
 * @return base 64 encoded data
 */
MIRA_BASE_EXPORT Buffer<uint8> base64Encode(const Buffer<uint8>& data);

/**
 * Base 64 decodes a base 64 encoded string.
 * @param[in] base64 Data to be decoded
 * @return decoded string
 */
MIRA_BASE_EXPORT std::string base64Decode(const std::string& base64);

/**
 * Base 64 decodes base 64 encoded data in a buffer.
 * @param[in] base64 Data to be decoded
 * @return decoded data
 */
MIRA_BASE_EXPORT Buffer<uint8> base64Decode(const Buffer<uint8>& base64);

///////////////////////////////////////////////////////////////////////////////

}

#endif
